@@ -1,8 +1,7 @@
 /**************************************************************************
  *   rcfile.c  --  This file is part of GNU nano.                         *
  *                                                                        *
- *   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,  *
- *   2010, 2011, 2013, 2014 Free Software Foundation, Inc.                *
+ *   Copyright (C) 2001-2011, 2013-2017 Free Software Foundation, Inc.    *
  *   Copyright (C) 2014 Mike Frysinger                                    *
  *   Copyright (C) 2014, 2015, 2016 Benno Schulenberg                     *
  *                                                                        *
@@ -31,7 +30,7 @@
 #include <unistd.h>
 #include <ctype.h>
 
-#ifndef DISABLE_NANORC
+#ifdef ENABLE_NANORC
 
 #ifndef RCFILE_NAME
 #define RCFILE_NAME ".nanorc"
@@ -45,8 +44,8 @@ static const rcoption rcopts[] = {
 #ifndef DISABLE_JUSTIFY
     {"brackets", 0},
 #endif
-    {"const", CONST_UPDATE},  /* deprecated form, remove in 2018 */
-    {"constantshow", CONST_UPDATE},
+    {"const", CONSTANT_SHOW},  /* deprecated form, remove in 2018 */
+    {"constantshow", CONSTANT_SHOW},
 #ifndef DISABLE_WRAPJUSTIFY
     {"fill", 0},
 #endif
@@ -54,10 +53,10 @@ static const rcoption rcopts[] = {
     {"historylog", HISTORYLOG},
 #endif
     {"morespace", MORE_SPACE},
-#ifndef DISABLE_MOUSE
+#ifdef ENABLE_MOUSE
     {"mouse", USE_MOUSE},
 #endif
-#ifndef DISABLE_MULTIBUFFER
+#ifdef ENABLE_MULTIBUFFER
     {"multibuffer", MULTIBUFFER},
 #endif
     {"nohelp", NO_HELP},
@@ -78,6 +77,7 @@ static const rcoption rcopts[] = {
     {"punct", 0},
     {"quotestr", 0},
 #endif
+    {"quickblank", QUICK_BLANK},
     {"rebinddelete", REBIND_DELETE},
     {"rebindkeypad", REBIND_KEYPAD},
     {"regexp", USE_REGEXP},
@@ -100,7 +100,6 @@ static const rcoption rcopts[] = {
     {"locking", LOCKING},
     {"matchbrackets", 0},
     {"noconvert", NO_CONVERT},
-    {"quickblank", QUICK_BLANK},
     {"quiet", QUIET},
     {"showcursor", SHOW_CURSOR},
     {"smarthome", SMART_HOME},
@@ -160,9 +159,9 @@ void rcfile_error(const char *msg, ...)
 
     fprintf(stderr, "\n");
 }
-#endif /* !DISABLE_NANORC */
+#endif /* ENABLE_NANORC */
 
-#if !defined(DISABLE_NANORC) || !defined(DISABLE_HISTORIES)
+#if defined(ENABLE_NANORC) || !defined(DISABLE_HISTORIES)
 /* Parse the next word from the string, null-terminate it, and return
  * a pointer to the first character after the null terminator.  The
  * returned pointer will point to '\0' if we hit the end of the line. */
@@ -182,9 +181,9 @@ char *parse_next_word(char *ptr)
 
     return ptr;
 }
-#endif /* !DISABLE_NANORC || !DISABLE_HISTORIES */
+#endif /* ENABLE_NANORC || !DISABLE_HISTORIES */
 
-#ifndef DISABLE_NANORC
+#ifdef ENABLE_NANORC
 /* Parse an argument, with optional quotes, after a keyword that takes
  * one.  If the next word starts with a ", we say that it ends with the
  * last " of the line.  Otherwise, we interpret it as usual, so that the
@@ -351,7 +350,9 @@ bool is_universal(void (*func))
 {
     if (func == do_left || func == do_right ||
 	func == do_home_void || func == do_end_void ||
+#ifndef NANO_TINY
 	func == do_prev_word_void || func == do_next_word_void ||
+#endif
 	func == do_verbatim_input || func == do_cut_text_void ||
 	func == do_delete || func == do_backspace ||
 	func == do_tab || func == do_enter)
@@ -1301,4 +1302,4 @@ void do_rcfiles(void)
     }
 }
 
-#endif /* !DISABLE_NANORC */
+#endif /* ENABLE_NANORC */
