@@ -202,7 +202,7 @@ int search_init(bool replacing, bool use_answer)
 	TOGGLE(USE_REGEXP);
 	backupstring = mallocstrcpy(backupstring, answer);
 	return 1;
-    } else if (func == do_replace || func == flip_replace_void) {
+    } else if (func == flip_replace) {
 	backupstring = mallocstrcpy(backupstring, answer);
 	return -2;	/* Call the opposite search function. */
     } else if (func == do_gotolinecolumn_void) {
@@ -234,8 +234,10 @@ int findnextstr(const char *needle, bool whole_word_only, bool have_region,
     size_t found_x;
 	/* The x coordinate of a found occurrence. */
     time_t lastkbcheck = time(NULL);
+	/* The time we last looked at the keyboard. */
 
-    enable_nodelay();
+    /* Set non-blocking input so that we can just peek for a Cancel. */
+    disable_waiting();
 
     if (begin == NULL)
 	came_full_circle = FALSE;
@@ -252,7 +254,7 @@ int findnextstr(const char *needle, bool whole_word_only, bool have_region,
 	    while (input) {
 		if (func_from_key(&input) == do_cancel) {
 		    statusbar(_("Cancelled"));
-		    disable_nodelay();
+		    enable_waiting();
 		    return -2;
 		}
 		input = parse_kbinput(NULL);
@@ -303,7 +305,7 @@ int findnextstr(const char *needle, bool whole_word_only, bool have_region,
 	/* If we're back at the beginning, then there is no needle. */
 	if (came_full_circle) {
 	    not_found_msg(needle);
-	    disable_nodelay();
+	    enable_waiting();
 	    return 0;
 	}
 
@@ -317,7 +319,7 @@ int findnextstr(const char *needle, bool whole_word_only, bool have_region,
 	 * but stop when spell-checking or replacing in a region. */
 	if (line == NULL) {
 	    if (whole_word_only || have_region) {
-		disable_nodelay();
+		enable_waiting();
 		return 0;
 	    }
 
@@ -343,16 +345,15 @@ int findnextstr(const char *needle, bool whole_word_only, bool have_region,
 
     found_x = found - line->data;
 
+    enable_waiting();
+
     /* Ensure that the found occurrence is not beyond the starting x. */
     if (came_full_circle && ((!ISSET(BACKWARDS_SEARCH) && found_x > begin_x) ||
 			(ISSET(BACKWARDS_SEARCH) && found_x < begin_x))) {
 	not_found_msg(needle);
-	disable_nodelay();
 	return 0;
     }
 
-    disable_nodelay();
-
     /* Set the current position to point at what we found. */
     openfile->current = line;
     openfile->current_x = found_x;
@@ -361,8 +362,11 @@ int findnextstr(const char *needle, bool whole_word_only, bool have_region,
     if (match_len != NULL)
 	*match_len = found_len;
 
-    if (feedback > 0)
+    /* Wipe the "Searching..." message and unset the suppression flag. */
+    if (feedback > 0) {
 	blank_statusbar();
+	suppress_cursorpos = FALSE;
+    }
 
     return 1;
 }
@@ -855,10 +859,10 @@ void do_gotolinecolumn(ssize_t line, ssize_t column, bool use_answer,
 	    return;
 	}
     } else {
-	if (line < 1)
+	if (line == 0)
 	    line = openfile->current->lineno;
 
-	if (column < 1)
+	if (column == 0)
 	    column = openfile->placewewant + 1;
     }
 
@@ -883,6 +887,12 @@ void do_gotolinecolumn(ssize_t line, ssize_t column, bool use_answer,
     openfile->current_x = actual_x(openfile->current->data, column - 1);
     openfile->placewewant = column - 1;
 
+#ifndef NANO_TINY
+    if (ISSET(SOFTWRAP) && openfile->placewewant / editwincols >
+			strlenpt(openfile->current->data) / editwincols)
+	openfile->placewewant = strlenpt(openfile->current->data);
+#endif
+
     /* When the position was manually given, center the target line. */
     if (interactive) {
 	adjust_viewport(CENTERING);
@@ -1224,7 +1234,7 @@ void get_history_older_void(void)
     ;
 }
 
-#ifndef DISABLE_TABCOMP
+#ifdef ENABLE_TABCOMP
 /* Move h to the next string that's a tab completion of the string s,
  * looking at only the first len characters of s, and return that
  * string.  If there isn't one, or if len is 0, don't move h and return
@@ -1277,5 +1287,5 @@ char *get_history_completion(filestruct **h, char *s, size_t len)
      * match, or len is 0.  Return s. */
     return (char *)s;
 }
-#endif /* !DISABLE_TABCOMP */
+#endif /* ENSABLE_TABCOMP */
 #endif /* !DISABLE_HISTORIES */
